#include <stdio.h>
#include <string.h>
#include <sys/stat.h> /* for fstat() */
#include <stdlib.h> /* for malloc() and free() */

#include "io.h"

char * readString(char * buffer, const unsigned short int maxlen)
{ /* fgets includes a newline at the end of the read string, so we remove it */
  fgets(buffer, maxlen, stdin);
  /* maximum string length will be maxlen - 2 because of \n and \0 */
  buffer[strlen(buffer) - 1] = '\0';
  return buffer;
}

int fileExists(const char * filename)
{ /* returns 1 if file exists, 0 if not */
  FILE * file = fopen(filename, "r");
  if (file == NULL) /* fopen() failed so file doesn't exist */
    return 0; /* meaning false, not success */
  else /* file was opened so close it and return true */
  {
    fclose(file);
    return 1;
  }
}

int fileSize(const char * filename)
{
  struct stat filestatus; /* will contain info about file */
  int filesize = 0;
  FILE * file = fopen(filename, "r"); /* open the file */

  if (file == NULL)
  {
    perror("fopen() failed in fileSize()");
    return -1;
  }

  if (fstat(fileno(file), &filestatus) == -1) /* get file status (and info) */
  {
    perror("fstat() failed in fileSize()");
    return -1;
  }

  filesize = filestatus.st_size; /* get filesize from file status */
  fclose(file);
  return filesize;
}

const char * readFile(const char * filename)
{
  int filesize = fileSize(filename);
  char * filecontents = NULL;
  FILE * file;

  if (filesize == -1)
    return NULL;

  file = fopen(filename, "r");
  if (file == NULL)
    return NULL;

  filecontents = (char *) malloc(filesize); /* allocate space for file data */
  if (fread(filecontents, 1, filesize, file) == -1) /* read whole file */
  {
    free(filecontents);
    return NULL;
  }

  fclose(file);
  return filecontents;
}

FILE * createEmptyFile(const char * filename)
{
  FILE * file;
  if ((file = fopen(filename, "w")) == NULL)
    return NULL;
  else
    return file;
}
