#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "xstring.h"
#include "url.h"

/* public functions */

Url * urlCreate(char * urlstr)
{
  Url * url = NULL;

  char * pPath = NULL;
  char * pStartMarker = NULL;
  char * pEndMarker = NULL;

  /* check for :// separator */

  pStartMarker = urlstr;
  pEndMarker = strstr(urlstr, "://");

  if (pEndMarker == NULL) /* no separator => invalid URL */
    return NULL;

  /* allocate memory for Url object */

  url = (Url *) malloc(sizeof(Url));
  url->protocol = NULL;
  url->host = NULL;
  url->port = 80;
  url->path = NULL;
  url->filename = NULL;
  url->query = NULL;
  url->id = NULL;

  /* get protocol part */

  url->protocol = xstrSubstr(pStartMarker, 0, pEndMarker - pStartMarker);

  /* get host part */

  pStartMarker = pEndMarker + 3; /* pStartMarker -> host */
  pEndMarker = strchr(pStartMarker, ':'); /* pStartMarker -> port if any */
  pPath = strchr(pStartMarker, '/');

  if (pEndMarker != NULL) /* there is a port after the host */
  {
    if (pPath == NULL) /* no path so : must be of port */
      url->host = xstrSubstr(pStartMarker, 0, pEndMarker - pStartMarker);
    else if (pEndMarker > pPath) /* : is in another field e.g. id */
      url->host = xstrSubstr(pStartMarker, 0, pPath - pStartMarker);
    else /* ok... : is that of port */
      url->host = xstrSubstr(pStartMarker, 0, pEndMarker - pStartMarker);
  }
  else /* there is no port after the host */
  {
    if (pPath == NULL) /* there is no path, so just get the host */
    {
      url->host = xstrSubstr(pStartMarker, 0, strlen(pStartMarker));
      return url;
    }
    else /* no port but yes path... so read host up to path */
      url->host = xstrSubstr(pStartMarker, 0, pPath - pStartMarker);
  }

  /* get port part */

  if (pEndMarker != NULL)
  {
    pEndMarker++; /* skip the : */
    if (pPath != NULL)
    {
      if (pEndMarker < pPath)
        url->port = atoi(pEndMarker);
    }
    else
      url->port = atoi(pEndMarker);
  }

  /* get path and other fields */

  if (pPath != NULL)
  {
    /* get path part */

    pStartMarker = pPath; /* pStartMarker -> pPath */
    pEndMarker = pStartMarker;

    while ((pEndMarker = strchr(pEndMarker, '/')) != NULL)
    {
      pStartMarker = pEndMarker; /* keeps track of previous slash */
      pEndMarker++; /* skip char found or it will get stuck in loop */
    }
    pEndMarker = pStartMarker + 1; /* pEndMarker -> byte after last slash */

    url->path = xstrSubstr(pPath, 0, pEndMarker - pPath);

    pStartMarker = pEndMarker; /* pStartMarker -> filename or whatever */

    /* get filename part */

    if (*pStartMarker != '?' && *pStartMarker != '#') /* filename exists */
    {
      pEndMarker = strchr(pStartMarker, '?');
      if (pEndMarker == NULL) /* no querystring */
        pEndMarker = strchr(pStartMarker, '#');
      if (pEndMarker == NULL)
        pEndMarker = pStartMarker + strlen(pStartMarker);

      url->filename = xstrSubstr(pStartMarker, 0, pEndMarker - pStartMarker);
    }

    pStartMarker = pEndMarker; /* pStartMarker -> next field */

    /* get querystring part */

    if (*pStartMarker == '?')
    {
      pStartMarker++; /* skip '?' */
      if (*pStartMarker != '#')
      {
        pEndMarker = strchr(pStartMarker, '#');
        if (pEndMarker == NULL)
          pEndMarker = pStartMarker + strlen(pStartMarker);

        url->query = xstrSubstr(pStartMarker, 0, pEndMarker - pStartMarker);
      }
    }

    pStartMarker = pEndMarker;

    /* get id part */

    if (*pStartMarker == '#')
    {
      pStartMarker++; /* skip '#' */
      if (*pStartMarker != '\0')
      {
        pEndMarker = pStartMarker + strlen(pStartMarker);

        url->id = xstrSubstr(pStartMarker, 0, pEndMarker - pStartMarker);
      }
    }
  }

  /* return Url object */

  return url;
}

char * urlGetProtocol(Url * url)
{
  if (url->protocol == NULL)
    return NULL;
  else
    return url->protocol->str;
}

char * urlGetHost(Url * url)
{
  if (url->host == NULL)
    return NULL;
  else
    return url->host->str;
}

int urlGetPort(Url * url)
{
  return url->port;
}

char * urlGetPath(Url * url)
{
  if (url->path == NULL)
    return NULL;
  else
    return url->path->str;
}

char * urlGetFilename(Url * url)
{
  if (url->filename == NULL)
    return NULL;
  else
    return url->filename->str;
}

char * urlGetQuery(Url * url)
{
  if (url->query == NULL)
    return NULL;
  else
    return url->query->str;
}

char * urlGetId(Url * url)
{
  if (url->id == NULL)
    return NULL;
  else
    return url->id->str;
}

void urlShow(Url * url)
{
  printf("%s://%s", url->protocol->str, url->host->str);
  if (url->port != -1)
    printf(":%d", url->port);
  if (url->path != NULL)
    printf("%s", url->path->str);
  if (url->filename != NULL)
    printf("%s", url->filename->str);
  if (url->query != NULL)
    printf("?%s", url->query->str);
  if (url->id != NULL)
    printf("#%s", url->id->str);
  printf("\n");
}

void urlShowFields(Url * url)
{
  if (url->protocol != NULL)
    printf("protocol: %s\n", url->protocol->str);
  else
    printf("protocol: (null)\n");

  if (url->host != NULL)
    printf("host:     %s\n", url->host->str);
  else
    printf("host:     (null)\n");

  if (url->port >= 0)
    printf("port:     %d\n", url->port);
  else
    printf("port:     (null)\n");

  if (url->path != NULL)
    printf("path:     %s\n", url->path->str);
  else
    printf("path:     (null)\n");

  if (url->filename != NULL)
    printf("filename: %s\n", url->filename->str);
  else
    printf("filename: (null)\n");

  if (url->query != NULL)
    printf("query:    %s\n", url->query->str);
  else
    printf("query:    (null)\n");

  if (url->id != NULL)
    printf("id:       %s\n", url->id->str);
  else
    printf("id:       (null)\n");
}

void urlDestroy(Url * url)
{
  if (url->protocol != NULL)
    free(url->protocol);
  if (url->host != NULL)
    free(url->host);
  if (url->path != NULL)
    free(url->path);
  if (url->filename != NULL)
    free(url->filename);
  if (url->query != NULL)
    free(url->query);
  if (url->id != NULL)
    free(url->id);

  free(url);
}
