#include <stdlib.h>
#include <string.h>

#include "xstring.h"

/** XString creation (must call xstrDestroy() to free them) */

XString * xstrCreate(char * str)
{
  return xstrSubstr(str, 0, 0);
}

XString * xstrSubstr(char * str, int offset, int len)
{
  XString * xstr = (XString *) malloc(sizeof(XString));
  int substrlen = (len == 0) ? strlen(str) : len;
  char * substr = str + offset;

  xstr->length = substrlen;
  xstr->str = (char *) malloc(substrlen + 1);
  strncpy(xstr->str, substr, substrlen);
  xstr->str[substrlen] = '\0';

  return xstr;
}

XString * xstrXSubstr(XString * xstr, int offset, int len)
{
  return xstrSubstr(xstr->str, offset, len);
}

/** XString manipulation */

void xstrAppend(XString * xstr, char * str)
{
  int strlength = strlen(str);
  int totallength = xstr->length + strlength;
  char * buffer = (char *) malloc(totallength + 1);

  strcpy(buffer, xstr->str);
  strcpy(buffer + xstr->length, str);
  buffer[totallength] = '\0';

  free(xstr->str);
  xstr->str = buffer;
  xstr->length = totallength;
}

/** XString querying */

char xstrCharAt(XString * xstr, int index)
{
  return xstr->str[index];
}

int xstrEquals(XString * xstr, char * str)
{
  if (strcmp(xstr->str, str) == 0)
    return 1;
  else
    return 0;
}

int xstrLength(XString * xstr)
{
  return xstr->length;
}

char * xstrString(XString * xstr)
{
  return xstr->str;
}

int xstrContains(XString * xstr, char c)
{
  int occurrences = 0;
  char * next = xstr->str;

  while ((next = strchr(next, c)) != NULL)
  {
    next++; /* skip char found or it will get stuck in loop */
    occurrences++;
  }

  return occurrences;
}

int xstrIndexOf(XString * xstr, char c)
{
  char * location = strchr(xstr->str, c);

  if (location == NULL)
    return -1;
  else
    return (location - xstr->str);
}

int xstrLastIndexOf(XString * xstr, char c)
{
  int i = strlen(xstr->str);
  while (i >= 0)
    if (xstr->str[--i] == c) 
      return i;
  return -1;
}

/** XString destruction */

void xstrDestroy(XString * xstr)
{
  free(xstr->str);
  free(xstr);
}
